Package: ridigbio
Title: Interface to the iDigBio Data API
Version: 0.4.1
Date: 2024-9-27
Encoding: UTF-8
Authors@R: c(person("Francois", "Michonneau", comment="Original Author", role=c("aut", "cph"),
  email="francois.michonneau@gmail.com"),
  person("Matthew", "Collins", comment="Original Author", role=c("aut"),
  email="mcollins@acis.ufl.edu"),
  person("Scott", "Chamberlain", role="ctb",
  email="myrmecocystus@gmail.com"),
  person("Kevin", "Love", role="ctb",
  email="klove@flmnh.ufl.edu"),
  person("Hem Nalini", "Morzaria-Luna", role="ctb",
  email="hemnalin.morzarialuna@noaa.gov"),
  person("Michelle L.", "Gaynor", role=c("ctb", "aut"),
  email="michellegaynor@ufl.edu"),
  person("Jesse Bennett", comment="Maintainer", role=c("cre"),
  email="j3553@ufl.edu"))
Description: An interface to iDigBio's search API that allows downloading
  specimen records. Searches are returned as a data.frame. Other functions
  such as the metadata end points return lists of information. iDigBio is a US
  project focused on digitizing and serving museum specimen collections on the
  web. See <https://www.idigbio.org> for information on iDigBio.
URL: https://github.com/iDigBio/ridigbio,
        https://idigbio.github.io/ridigbio/
BugReports: https://github.com/iDigBio/ridigbio/issues
Depends: R (>= 3.0.1)
Imports: stats, plyr, httr, jsonlite, leaflet, kableExtra, tidyverse,
        cowplot
License: MIT + file LICENSE
Suggests: testthat, markdown, knitr, utils, rmarkdown
VignetteBuilder: knitr
Repository: CRAN
RoxygenNote: 7.3.1
NeedsCompilation: no
Packaged: 2024-09-30 17:28:06 UTC; jessebennett
Author: Francois Michonneau [aut, cph] (Original Author),
  Matthew Collins [aut] (Original Author),
  Scott Chamberlain [ctb],
  Kevin Love [ctb],
  Hem Nalini Morzaria-Luna [ctb],
  Michelle L. Gaynor [ctb, aut],
  Jesse Bennett [cre] (Maintainer)
Maintainer: Jesse Bennett <j3553@ufl.edu>
Date/Publication: 2024-10-01 02:10:03 UTC
Built: R 4.6.0; ; 2026-04-30 19:55:52 UTC; unix
